@@ -8,15 +8,19 @@
 pub
 mod prelude {
     pub use crate::{
+        exit_polonius,
         polonius,
-        polonius_return,
         polonius_break,
+        polonius_continue,
+        polonius_loop,
+        polonius_return,
     };
 }
 
 /// See the [top-level docs][crate] for more info.
 pub
-trait WithLifetime<'lt, Bound = &'lt Self> {
+trait WithLifetime<'lt> { // Note: the `&'lt Self` implicit bound hack is,
+                          // for once, unnecessary.
     type T;
 }
 
@@ -34,7 +38,7 @@ where
 
 /// See the [top-level docs][crate] for more info.
 pub
-fn polonius<Ret : HKT, State : ?Sized, Err, F> (
+fn polonius<Ret : ?Sized + HKT, State : ?Sized, Err, F> (
     state: &mut State,
     branch: F,
 ) -> Result<
@@ -91,7 +95,7 @@ let x = polonius!(|a_mut_binding| -> SomeRetType<'polonius> {
         polonius_return!(some_dependent_type);
     }
     if some_other_cond() {
-        polonius_break!(42);
+        exit_polonius!(42);
         unreachable!();
     }
     42
@@ -104,10 +108,7 @@ stuff(a_mut_binding) // macro gave it back
 ///
 /// ### Generic parameters
 ///
-/// Sometimes, the return value of that `polonius!` closure may need to refer to
-/// generic parameters. When that's the case, you'll need to tell the macro
-/// about those, using `<generics…>` syntax before the `|…| ->` part of the
-/// macro input:
+/// They now Just Work™.
 ///
 /** ```rust
 use ::polonius_the_crab::prelude::*;
@@ -121,12 +122,7 @@ where
     K : ::core::hash::Hash + Eq + Clone,
     V : ::core::fmt::Debug,
 {
-    // No need to provide `K`, since not part of the return type;
-    // nor to provide the bounds for `V`, except for:
-    //   - nested borrows (if ever),
-    //   - `?Sized` (every now and then)
-    //   - or to be able to name a bound-provided associated type (super rare!)
-    polonius!(<'v, V : 'v + ?Sized> |map| -> &'polonius &'v V {
+    polonius!(|map| -> &'polonius &'v V {
         if let Some(v) = map.get(key) {
             dbg!(v); // Even though `Debug` is not provided to the signature, it is available to the body.
             polonius_return!(v);
@@ -138,69 +134,13 @@ where
 ``` */
 #[macro_export]
 macro_rules! polonius {(
-    $(
-        <
-            $( $(
-                $lt:lifetime
-            ),+ $(,)? )?
-            $( $(
-                $T:ident $(:
-                    $( $super:lifetime + )?
-                    $( ?$Sized:ident )?
-                    $( + )?
-                    $( $Bound:path )?
-                )?
-            ),+ $(,)? )?
-        >
-    )?
-    |$var:ident| -> $Ret:ty $body:block
+    |$var:ident $(,)?| -> $Ret:ty
+        $body:block
+    $(,)?
 ) => ({
-    #[allow(nonstandard_style)]
-    struct __polonius_the_crab_Ret <$(
-        $($(
-            $lt
-        ,)+)?
-        $($(
-            $T $(:
-                $( $super + )?
-                $( ?$Sized + )?
-                $( $Bound )?
-            )?
-        ,)+)?
-    )?> (
-        *mut Self,
-    );
-
-    impl<
-        'polonius,
-        $(
-            $($(
-                $lt
-            ,)+)?
-            $($(
-                $T $(:
-                    $( $super + )?
-                    $( ?$Sized + )?
-                    $( $Bound )?
-                )?
-            ,)+)?
-        )?
-    >
-        $crate::WithLifetime<'polonius>
-    for
-        __polonius_the_crab_Ret<$(
-            $($($lt ,)+)?
-            $($($T ,)+)?
-        )?>
-    {
-        type T = $Ret;
-    }
-
     match
         $crate::polonius::<
-            __polonius_the_crab_Ret<$(
-                $($($T ,)+)?
-            )?>,
+            dyn for<'polonius> $crate::WithLifetime<'polonius, T = $Ret>,
             _,
             _,
             _,
@@ -238,8 +178,133 @@ macro_rules! polonius_return {( $e:expr $(,)? ) => (
 
 /// See [`polonius!`] for more info.
 #[macro_export]
+macro_rules! exit_polonius {( $($e:expr $(,)?)? ) => (
+    return $crate::ඞ::core::result::Result::Err(
+        ($($e ,)? (),).0
+    )
+)}
+
+/// Convenience support for the `loop { … polonius!(…) }` pattern.
+///
+/// ### Example
+///
+/** ```rust
+#![forbid(unsafe_code)]
+use {
+    ::polonius_the_crab::{
+        prelude::*,
+    },
+    ::std::{
+        collections::HashMap,
+    },
+};
+
+enum Value {
+    Alive(i32),
+    Daed,
+}
+
+/// Notice how this example, *despite its usage of the fancy `.entry()` API
+/// of `HashMap`s*, still needs `polonius_the_crab` to express this logic!
+fn get_first_alive_from_base_or_insert (
+    mut map: &'_ mut HashMap<usize, Value>,
+    base: usize,
+    default_value: i32,
+) -> &'_ i32
+{
+    let mut idx = base;
+    // (loop {
+    polonius_loop!(|map| -> &'polonius i32 {
+        use ::std::collections::hash_map::*;
+        // return(
+        polonius_return!(
+            match map.entry(idx) {
+                | Entry::Occupied(entry) => match entry.into_mut() {
+                    // Found a value!
+                    | &mut Value::Alive(ref val) => val,
+                    // "tombstone", keep searching
+                    | &mut Value::Daed => {
+                        idx += 1;
+                        // continue;
+                        polonius_continue!();
+                    },
+                },
+                | Entry::Vacant(slot) => match slot.insert(Value::Alive(default_value)) {
+                    | &mut Value::Alive(ref val) => val,
+                    | &mut Value::Daed => unreachable!(),
+                },
+            }
+        );
+    })
+}
+``` */
+///
+/// <details><summary>Error message without <code>polonius</code></summary>
+///
+/** ```console
+ error[E0499]: cannot borrow `*map` as mutable more than once at a time
+   --> src/lib.rs:222:18
+    |
+ 22 | mut map: &'_ mut HashMap<usize, Value>,
+    |          - let's call the lifetime of this reference `'1`
+ ...
+ 33 |     match map.entry(idx) {
+    |           ^^^ `*map` was mutably borrowed here in the previous iteration of the loop
+ ...
+ 45 |         | &mut Value::Alive(ref val) => val,
+    |                                         --- returning this value requires that `*map` be borrowed for `'1`
+``` */
+///
+/// ___
+///
+/// </details>
+#[macro_export]
+macro_rules! polonius_loop {(
+    | $var:ident $(,)? | -> $Ret:ty
+        $body:block
+    $(,)?
+) => (
+    loop {
+        match $crate::polonius!(
+            | $var | -> $Ret {
+                let () =
+                    if true
+                        $body
+                    else {
+                        // avoid a dead-code warning
+                        $crate::ඞ::core::option::Option::None.unwrap()
+                    }
+                ;
+                $crate::polonius_continue!();
+            },
+        )
+        {
+            | $crate::ඞ::core::ops::ControlFlow::Break(value) => break value,
+            | $crate::ඞ::core::ops::ControlFlow::Continue(()) => continue,
+        }
+    }
+)}
+
+/// `break` a **non-dependent value** out of a [`polonius_loop!`].
+///
+///   - (when the value to `break` with is dependent, then the necessary pattern
+///     is not `loop { polonius!(…) }` (what [`polonius_loop!`] stands for) but
+///     `polonius!(… let it = loop { … break … }; …)`).
+#[macro_export]
 macro_rules! polonius_break {( $($e:expr $(,)?)? ) => (
-    return $crate::ඞ::core::result::Result::Err(($($e)?))
+    return $crate::ඞ::core::result::Result::Err(
+        $crate::ඞ::core::ops::ControlFlow::Break(
+            ($($e ,)? () ,).0
+        )
+    )
+)}
+
+/// `continue` to the next iteration of a [`polonius_loop!`].
+#[macro_export]
+macro_rules! polonius_continue {() => (
+    return $crate::ඞ::core::result::Result::Err(
+        $crate::ඞ::core::ops::ControlFlow::<_>::Continue(())
+    )
 )}
 
 // macro internals
